/* rxtxdll.h
 *
 *
 * Author: Manuel Sickert
 * Copyright 2009, Manuel Sickert, Prgen, Germany
 */

#ifndef RXTXDLL_H
#define RXTXDLL_H

/* Minimum supported system is Windows 95 */
#define _WIN32_WINDOWS  0x0500
#define _WIN32_WINNT  0x0500
#define WINVER  0x0500


#include <stdio.h>
#include <windows.h>


#ifdef __cplusplus
#define EXPORT extern "C" __declspec (dllexport)
#else
#define EXPORT __declspec (dllexport)
#endif

/*** PUBLIC INTERFACE **********************************************/

/* GetIntSize() returns the size (in bytes) of the integer type */
EXPORT int __stdcall GetIntSize();

/* ConfigPort opens the specified port (port enumeration starts with 1) in non-overlapped mode
 * (synchronous mode) for read and write access. Once the port is opened it cannot be shared with other applications
 * (exclusive access for the calling process). Data transfer is managed without handshaking.
 * Parameters:
 * port: COM port number, counting starts at one.
 * baud: baudrate, e.g. 9600, 38400, 115200
 * databits: number of data bits between 5 and 8
 * parity: 0 = no parity, 1 = odd parity, 2 = even parity, 3 = mark parity, 4 = space parity
 * stopbits: 0 = one stop bit, 1 = 1.5 stop bits, 2 = 2 stop bits
 * Please note the following restrictions:
 *   - The number of data bits must be 5 to 8 bits. 
 *   - The use of 5 data bits with 2 stop bits is an invalid combination, 
 *     as is 6, 7, or 8 data bits with 1.5 stop bits. 
 *
 * On success ConfigPort returns a HANDLE to the specified port.
 * On failure ConfigPort returns zero.
 */
EXPORT HANDLE __stdcall ConfigPort(int port, long baud, int databits, char parity, char stopbits);

EXPORT int __stdcall ClosePort(HANDLE h);


/* Read the specified number of bytes
 * Return values:
 * >0: the number of bytes read
 * -1: reached EOF
 * -2: ReadBytes failed for another reason (e.g. port closed)
 */  
EXPORT int __stdcall ReadBytes(HANDLE h, char* buf, DWORD numBytesToRead);

/* buf is supposed to be a char array of length (maxBytesToRead+1) as we append '0'
 * to the end of the string.
 * A line has to be terminated by a '\n' (newline, 0x0a).
 * ReadLine returns the number of bytes read.
 */
EXPORT int __stdcall ReadLine(HANDLE h, char* buf, DWORD maxBytesToRead);

/* WriteByte returns the number of bytes written */
EXPORT int __stdcall WriteBytes(HANDLE h, char* buf, DWORD numBytesToWrite);

/* SetTimeouts is used to set data transfer timeouts for read and write operations.
 * Parameters:
 * ReadIntervalTimeout 
 *    Maximum time allowed to elapse between the arrival of two bytes on the communications 
 *    line, in milliseconds. During a ReadFile operation, the time period begins when 
 *    the first byte is received. If the interval between the arrival of any two bytes 
 *    exceeds this amount, the ReadFile operation is completed and any buffered data 
 *    is returned. A value of zero indicates that interval time-outs are not used. 
 *    A value of MAXDWORD, combined with zero values for both the ReadTotalTimeoutConstant 
 *    and ReadTotalTimeoutMultiplier members, specifies that the read operation is to return 
 *    immediately with the bytes that have already been received, even if no bytes have been received.
 *
 * ReadTotalTimeoutMultiplier 
 *    Multiplier used to calculate the total time-out period for read operations, in milliseconds. 
 *    For each read operation, this value is multiplied by the requested number of bytes to be read. 
 *
 * ReadTotalTimeoutConstant 
 *    Constant used to calculate the total time-out period for read operations, in milliseconds. 
 *    For each read operation, this value is added to the product of the ReadTotalTimeoutMultiplier 
 *    member and the requested number of bytes. 
 *    A value of zero for both the ReadTotalTimeoutMultiplier and ReadTotalTimeoutConstant 
 *    members indicates that total time-outs are not used for read operations.
 *
 * WriteTotalTimeoutMultiplier 
 *    Multiplier used to calculate the total time-out period for write operations, in milliseconds. 
 *    For each write operation, this value is multiplied by the number of bytes to be written. 
 *
 * WriteTotalTimeoutConstant 
 *    Constant used to calculate the total time-out period for write operations, in milliseconds. 
 *    For each write operation, this value is added to the product of the WriteTotalTimeoutMultiplier 
 *    member and the number of bytes to be written. 
 *    A value of zero for both the WriteTotalTimeoutMultiplier and WriteTotalTimeoutConstant members 
 *    indicates that total time-outs are not used for write operations.
 */
EXPORT int __stdcall SetTimeouts(HANDLE h, int ReadIntervalTimeout, int ReadTotalTimeoutMultiplier, int ReadTotalTimeoutConstant, 
	int WriteTotalTimeoutMultiplier, int WriteTotalTimeoutConstant);
	
EXPORT int __stdcall PurgeRXBuf(HANDLE h);
EXPORT int __stdcall PurgeTXBuf(HANDLE h);
/*******************************************************************/

#endif
